==> teleportd-api-0.1.3.tgz: package/lib/teleportd.js <==
// Copyright teleportd Ltd
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

var util = require('util');
var crypto = require('crypto');
var http = require('http');
var qs = require('querystring');
var buf = require('buffer');
var events = require('events');

/**
 * Dynamic Parser Object borrowed to twitter-node
 */
var Parser = function Parser() {    
  events.EventEmitter.call(this);
  this.buffer = '';
  return this;
};

// The parser emits events!
Parser.prototype = Object.create(events.EventEmitter.prototype);
Parser.END        = '\r\n';
Parser.END_LENGTH = 2;

Parser.prototype.receive = function receive(buffer) {    
  this.buffer += buffer.toString('utf8');
  var index, json;    
  // We have END?
  while ((index = this.buffer.indexOf(Parser.END)) > -1) {
    json = this.buffer.slice(0, index);
    this.buffer = this.buffer.slice(index + Parser.END_LENGTH);
    if (json.length > 0) {    
      try {
	json = JSON.parse(json);
	this.emit('object', json);
      } catch (error) {
	this.emit('error', error);
      }
    }
  }
};


/**
 * Teleportd API Wrapper Object
 * 
 * @extends {}
 * 
 * @param spec {apikey}
 */
var teleportd = function(spec, my) {
  my = my || {};
  
  my.streams = {};
  my.nextsid = 1;  

  my.apikey = spec.apikey || "INVALID";

  // public
  var search; /* search({loc, string, period, window, skip}, function(array) {...}); */
  var stream; /* stream({loc, string}, function(pic) {...}); */
  var stop;   /* stop(sid); */
  var get;    /* get(sha|shrt, function(pic) {...}); */

  // private
  var build;

  var that = {};

  /**
   * builds the get query
   * @param spec search spec
   * @return options 
   */
  build = function(spec, endpoint) {
    var headers = {'User-Agent': 'NodeJS teleportd-api Driver v0.1.1'};
    
    var q = { apikey: my.apikey };
    // parameters validation
    if(Array.isArray(spec.loc) && spec.loc.length === 4)       // loc     [stream|search]
      q.loc = JSON.stringify(spec.loc);
    if(typeof spec.string === 'string')                        // string  [stream|search]
      q.string = spec.string;
    if(Array.isArray(spec.period) && spec.period.length === 2) // period  [search]
      q.period = JSON.stringify(spec.period);
    if(typeof spec.window  === 'number')                       // window  [search]
      q.window = spec.window;
    if(typeof spec.skip === 'number')                          // skip    [search]
      q.skip = spec.skip;

    if(typeof spec.sha === 'string')                           // sha  [get]
      q.sha = spec.sha;
    if(typeof spec.shrt === 'string')                          // shrt [get]
      q.shrt = spec.shrt;

    var options = { host: 'v1.api.teleportd.com',
	    	    port: 8080,
	    	    path: '/' + endpoint + '?' + qs.stringify(q),
	    	    headers: headers };

    return options;    
  };

  /**
   * Performs a search and returns the array of pic received
   * @param spec {loc, string, period, window, skip}
   * @param cb   callback function
   */
  search = function(spec, cb) {
    http.get(build(spec, 'search'), function(res) {
	       var body = '[';
	       res.on('data', function(chunk) {
			body += chunk;
		      });
	       res.on('end', function() {
			// hackerish way to get an array
			body = body.substr(0, body.length - 1);
			body += ']';
			cb(JSON.parse(body.replace(/\r\n/g, ',')));
		      });
	     });
  };
  
  /**
   * Starts a stream search and calls cb on each pic received
   * @param spec {loc, string}
   * @param cb   callback function
   * @return sid stream id
   */
  stream = function(spec, cb, id) {
    var sid = my.nextsid++;       
    if(typeof id !== 'undefined')
      sid = id;      
    if(typeof my.streams[sid] === 'undefined')
      my.streams[sid] = { error: 0 };
    
    var restart = function() {
      my.streams[sid].error++;
      if(my.streams[sid].error > 5)
	my.streams[sid].error = 5;
      console.log('STREAM RESTART COUNT: ' + (my.streams[sid].error - 1) + ' ' + (1000 * Math.pow(2, my.streams[sid].error - 1)));
      setTimeout(function() {			      
		   stream(spec, cb, sid);
		 }, 1000 * Math.pow(2, my.streams[sid].error - 1));		 
    };	       

    http.get(build(spec, 'stream'), function(res) {
	       var parser = new Parser();
	       
	       my.streams[sid].res = res;
	       my.streams[sid].parser = parser;		 
	       
	       
	       parser.on('object', function(pic) {
			   my.streams[sid].error = 0;
			   cb(pic);
			 });
	       
	       res.on('data', function(chunk) {
			parser.receive(chunk);
		      });
	       res.on('end', function() {
			restart();
		      });	       
	       res.on('error', function(e) {
			restart();
		      });
	     }).on('error', function(e) {
		     restart();
		   });
    
    return sid;
  };

  /**
   * Stops a stream or all the stream
   * @param sid stream id or null for all streams
   */ 
  stop = function(sid) {
    if(sid && my.streams[sid]) {
      my.streams[sid].res.destroy();
      delete my.streams[sid];
    }
    else if(typeof sid === 'undefined') {
      for(var s in my.streams) {
	if(my.streams.hasOwnProperty(s))
	  my.streams[s].res.destroy();
      }
      my.streams = {};
    }
  };

  /**
   * Retrieves detailed information about a particular pic
   * @param ss sha or shrt
   */
  get = function(ss, cb) {
    var spec = {};
    if(typeof ss === 'string') {
      if(ss.length === 40)
	spec.sha = ss;
      else
	spec.shrt = ss;
    }
    http.get(build(spec, 'get'), function(res) {
	       var body = '';
	       res.on('data', function(chunk) {
			body += chunk;
		      });
	       res.on('end', function() {
			var res = JSON.parse(body);
			if(res.status === 'OK')
			  cb(res.body);
			else
			  cb();
		      });
	     });	       
  };


  // exposed methods
  that.search = search;
  that.stream = stream;
  that.stop = stop;
  that.get = get;

  return that;
};

exports.teleportd = teleportd;

==> teleportd-api-0.1.6.tgz: package/lib/teleportd.js <==
// Copyright teleportd Ltd
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

var util = require('util');
var crypto = require('crypto');
var http = require('http');
var qs = require('querystring');
var buf = require('buffer');
var events = require('events');

/**
 * Dynamic Parser Object borrowed to twitter-node
 */
var Parser = function Parser() {    
  events.EventEmitter.call(this);
  this.buffer = '';
  return this;
};

// The parser emits events!
Parser.prototype = Object.create(events.EventEmitter.prototype);
Parser.END        = '\r\n';
Parser.END_LENGTH = 2;

Parser.prototype.receive = function receive(buffer) {    
  this.buffer += buffer.toString('utf8');
  var index, json;    
  // We have END?
  while ((index = this.buffer.indexOf(Parser.END)) > -1) {
    json = this.buffer.slice(0, index);
    this.buffer = this.buffer.slice(index + Parser.END_LENGTH);
    if (json.length > 0) {    
      try {
	json = JSON.parse(json);
	this.emit('object', json);
      } catch (error) {
	this.emit('error', error);
      }
    }
  }
};


/**
 * Teleportd API Wrapper Object
 * 
 * @extends {}
 * 
 * @param spec {apikey}
 */
var teleportd = function(spec, my) {
  my = my || {};
  
  my.streams = {};
  my.nextsid = 1;  

  my.apikey = spec.apikey || "INVALID";

  // public
  var search; /* search({loc, string, period, window, skip}, function(array) {...}); */
  var stream; /* stream({loc, string}, function(pic) {...}); */
  var stop;   /* stop(sid); */
  var get;    /* get(sha|shrt, function(pic) {...}); */

  // private
  var build;

  var that = {};

  /**
   * builds the get query
   * @param spec search spec
   * @return options 
   */
  build = function(spec, endpoint) {
    var headers = {'User-Agent': 'NodeJS teleportd-api Driver v0.1.1'};
    
    var q = { apikey: my.apikey };
    // parameters validation
    if(Array.isArray(spec.loc) && spec.loc.length === 4)       // loc     [stream|search]
      q.loc = JSON.stringify(spec.loc);
    if(typeof spec.string === 'string')                        // string  [stream|search]
      q.string = spec.string;
    if(Array.isArray(spec.period) && spec.period.length === 2) // period  [search]
      q.period = JSON.stringify(spec.period);
    if(typeof spec.window  === 'number')                       // window  [search]
      q.window = spec.window;
    if(typeof spec.skip === 'number')                          // skip    [search]
      q.skip = spec.skip;

    if(typeof spec.sha === 'string')                           // sha  [get]
      q.sha = spec.sha;
    if(typeof spec.shrt === 'string')                          // shrt [get]
      q.shrt = spec.shrt;

    var options = { host: 'v1.api.teleportd.com',
	    	    port: 8080,
	    	    path: '/' + endpoint + '?' + qs.stringify(q),
	    	    headers: headers };

    return options;    
  };

  /**
   * Performs a search and returns the array of pic received
   * @param spec {loc, string, period, window, skip}
   * @param cb   callback function
   */
  search = function(spec, cb) {
    http.get(build(spec, 'search'), function(res) {
	       var body = '[';
	       res.on('data', function(chunk) {
			body += chunk;
		      });
	       res.on('end', function() {
			// hackerish way to get an array
			body = body.substr(0, body.length - 1);
			body += ']';
			cb(JSON.parse(body.replace(/\r\n/g, ',')));
		      });
	     });
  };
  
  /**
   * Starts a stream search and calls cb on each pic received
   * @param spec {loc, string}
   * @param cb   callback function
   * @return sid stream id
   */
  stream = function(spec, cb, id) {
    var sid = my.nextsid++;       
    if(typeof id !== 'undefined')
      sid = id;      
    if(typeof my.streams[sid] === 'undefined')
      my.streams[sid] = { error: 0 };
    
    var restart = function() {
      my.streams[sid].error++;
      if(my.streams[sid].error > 5)
	my.streams[sid].error = 5;
      util.debug('STREAM RESTART COUNT: ' + (my.streams[sid].error - 1) + ' ' + (1000 * Math.pow(2, my.streams[sid].error - 1)));
      setTimeout(function() {			      
		   stream(spec, cb, sid);
		 }, 1000 * Math.pow(2, my.streams[sid].error - 1));		 
    };	       

    http.get(build(spec, 'stream'), function(res) {
	       var parser = new Parser();
	       
	       my.streams[sid].res = res;
	       my.streams[sid].parser = parser;		 
	       
	       
	       parser.on('object', function(pic) {
			   my.streams[sid].error = 0;
			   cb(pic);
			 });
	       
	       res.on('data', function(chunk) {
			parser.receive(chunk);
		      });
	       res.on('end', function() {
			restart();
		      });	       
	       res.on('error', function(e) {
			restart();
		      });
	       res.connection.on('close', function(e) {
				   restart();
				 });
	       
	     }).on('error', function(e) {
		     restart();
		   });
    
    return sid;
  };

  /**
   * Stops a stream or all the stream
   * @param sid stream id or null for all streams
   */ 
  stop = function(sid) {
    if(sid && my.streams[sid]) {
      my.streams[sid].res.destroy();
      delete my.streams[sid];
    }
    else if(typeof sid === 'undefined') {
      for(var s in my.streams) {
	if(my.streams.hasOwnProperty(s))
	  my.streams[s].res.destroy();
      }
      my.streams = {};
    }
  };

  /**
   * Retrieves detailed information about a particular pic
   * @param ss sha or shrt
   */
  get = function(ss, cb) {
    var spec = {};
    if(typeof ss === 'string') {
      if(ss.length === 40)
	spec.sha = ss;
      else
	spec.shrt = ss;
    }
    http.get(build(spec, 'get'), function(res) {
	       var body = '';
	       res.on('data', function(chunk) {
			body += chunk;
		      });
	       res.on('end', function() {
			var res = JSON.parse(body);
			if(res.status === 'OK')
			  cb(res.body);
			else
			  cb();
		      });
	     });	       
  };


  // exposed methods
  that.search = search;
  that.stream = stream;
  that.stop = stop;
  that.get = get;

  return that;
};

exports.teleportd = teleportd;
